/*  

    Built using F.Belair and R.Unger's roboCanvas

*/


import java.awt.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.awt.image.*;

class MyCanvas extends Canvas 
{

private Graph graphToDraw;
  
private ProjectedNode translateBy;

  // stuff for double buffering
private Dimension buffSize = null;
private Image image = null;
private Graphics buffer = null;

  int bufferWidth = 0;
  int bufferHeight = 0;
  int imageWidth = 0;
  int imageHeight = 0;
  
  // Constructor
public MyCanvas()
  {
    super();
    graphToDraw = null;
    setBackground(Color.white);
    image = null;
    translateBy = null;
    
    buffSize = getSize();

    //System.out.println("Canvas created...");
  }
  
public void init()
  {
    reallocateBuffer();
  }
  
public void initGraph(Graph graph)
  {
    graphToDraw = graph;
  }

  
public void update(Graphics g)
  {
    //System.out.println("Calling paint");
    paint(g);
  }
  
  
  // This is what I call to update my canvas
public void updateCanvas()
  {
    if (getSize().equals(buffSize)) {
      //System.out.println("Calling drawloop");
      drawLoop();
    }

    //System.out.println("Calling repaint");
    //repaint();
    
    Graphics g = this.getGraphics();
    update(g);
  }


  // called by AWT.  I should just call repaint() ie component.repaint()
public void paint(Graphics g)
  {
    if (image == null)
      {
	System.err.println("NULL image, can't paint on NULL image");
	//return;
	reallocateBuffer();
	drawLoop();
      }
    
    if (!getSize().equals(buffSize)) // buffer is the wrong size so reallocate
      {
	System.err.println("Buffer wrong size, reallocating");
	reallocateBuffer();
	drawLoop();
      }
    
    // copy the buffer to the screen.  This returns immediately!!!
    //System.out.println("Calling drawImage");
    g.drawImage(image, 0, 0, this);
  }


public boolean imageUpdate(Image img, int infoflags, int x, int y, int height)
  {

    System.out.println("imageUpdate called");
    if ((infoflags & ImageObserver.ALLBITS) != 0)
      {
	// image is complete
	System.out.println("imageUpdate has all info");
	repaint();
	return false;
      }
    return true; // need more info
  }

public void reallocateBuffer()
  {
    if (buffer != null)
      buffer.dispose();  // if it exists dump it
    
    if (image != null)
      image.flush();     // if it exists flush it
    
    // reallocation
    buffSize = getSize();

    if (buffSize.width == 0 || buffSize.height == 0)
      {
	System.err.println("BuffSize is "+buffSize+" can't reallocate");
	return;
      }

    image = createImage(buffSize.width, buffSize.height);
    buffer = image.getGraphics();
    translateBy = new ProjectedNode(buffSize.width/2,buffSize.height/2,"origin");
    
    //System.out.println("Reallocating buffer size: "+buffSize);

  }

public void drawLoop()
  {
    if (image == null || buffer == null)
      reallocateBuffer();
    
    eraseBuffer();
    
    if (graphToDraw == null)
      {
	System.err.println("Nothing to draw");
	return;
      }
    

    ProjectedNode center = (ProjectedNode)graphToDraw.projectedNodes.get(graphToDraw.avgNode.name);
    //System.err.println("center vector = "+center);
    ProjectedNode centerN = center.scaleNode(-1.0);
    
    
    for (int i=0; i<graphToDraw.edges.size();i++)
      {
	Edge edge = (Edge)graphToDraw.edges.elementAt(i);
	Node one = edge.GetFirstNode();
	Node two = edge.GetSecondNode();
	ProjectedNode n1 = (ProjectedNode)graphToDraw.projectedNodes.get(one.name);	
	ProjectedNode n2 = (ProjectedNode)graphToDraw.projectedNodes.get(two.name);
	// Translate to canvas origin
	ProjectedNode o1 = n1.addNode(centerN);
	ProjectedNode o2 = n2.addNode(centerN);
	// scale
	ProjectedNode s1 = o1.scaleNode(3.0);
	ProjectedNode s2 = o2.scaleNode(3.0);
	// Translate to center of canvas
	ProjectedNode tn1 = s1.addNode(translateBy);
	ProjectedNode tn2 = s2.addNode(translateBy);

	drawNode((int)tn1.x,(int)tn1.y);
	drawNode((int)tn2.x,(int)tn2.y);
	//System.out.println("Drawing edge from "+tn1.name+" to "+tn2.name);
	drawEdge((int)tn1.x,(int)tn1.y,(int)tn2.x,(int)tn2.y);
      }
    
    //System.err.println("Buffer = "+buffer);

  }


public void eraseBuffer()
  {

    //Rectangle r = buffer.getClipBounds();
    
    //if (r == null)
    //  {
    //	System.err.println("eraseBuffer---->"+r);
    //return;
    //}


    //System.out.println("buffSize = "+buffSize);

    buffer.clearRect(0,0,buffSize.width,buffSize.height);
  }

public void drawNode(int x, int y)
  {
    buffer.setColor(Color.red);
    buffer.fillOval(x-2,y-2,4,4);
    buffer.setColor(Color.black);
    buffer.drawOval(x-2,y-2,4,4);
  }

public void drawEdge(int x1, int y1, int x2, int y2)
  {
    buffer.setColor(Color.blue);
    buffer.drawLine(x1,y1,x2,y2);
  }
  
}












