// This class spawns a thread to rotate the wireframe and stop it

class Rotate extends Thread
{
  
private Graph wireFrame;
private MyCanvas myCanvas;
  static final double PI = 3.1415927;

  
public Rotate(Graph inGraph, MyCanvas inCanvas)
  {
    wireFrame = inGraph;
    myCanvas = inCanvas;
  }
  
public void rotate(double thetad, double phid, double norm)
  {
    Node newCamera = new Node(norm*Math.cos(degToRad(thetad))*Math.cos(degToRad(phid)),
			      norm*Math.cos(degToRad(thetad))*Math.sin(degToRad(phid)),
			      norm*Math.sin(degToRad(thetad)),
			      "newCamera");
    showView(newCamera);
  }


public void run()
  {
    double theta = 0;
    double phi = 0;

    double thetad = 0;
    double phid = 0;

    // get any best viewing vector
    Node n = (Node)wireFrame.bestVectors.elementAt(0);
    
    if (n == null)
      {
	System.err.println("No camera available");
	return;
      }

    Node camera = wireFrame.avgNode.addNode(n.scaleNode(10));

    double norm = camera.norm();
    double sendNorm = norm;
    // compute theta and phi (euler angles)
    theta = Math.asin(camera.z/norm);
    // planar norm
    double pnorm = norm*Math.cos(theta); 
    phi = Math.asin(camera.y/pnorm);
    
    thetad = radToDeg(theta);
    phid = radToDeg(phi);


    while (true)
      {
	rotate(thetad, phid, sendNorm);
	try 
	  {
	    sleep(10);
	  }
	catch (InterruptedException e)
	  {}
	thetad = (thetad + 1.5) % 360;
	phid = (phid - 1) % 360;
      }
  }

public double radToDeg(double rad)
  {
    return 180*rad/PI;
  }
  
public double degToRad(double deg)
  {
    return deg*PI/180;
  }


public void showView(Node n)
  {
    if (wireFrame.avgNode == null)
      {
	//display.append("CHOOSE A SHAPE FIRST\n");
	return;
      }
    wireFrame.showView(n);
    myCanvas.initGraph(wireFrame);
    myCanvas.updateCanvas();	    
  }


}
